/**
 * @file main.c
 * @brief Здесь входим в программу
 * @author Маркин Алексей (alexanius.ru)
 */

/**
 * @mainpage Введение
 *
 * Это пятый урок. В нём мы строим дерево разбора на основе нашей грамматики.
 */

#include <stdio.h>

#include "x-lexer.h"
#include "x-grammar.h"
#include "x-node.h"

/**
 * @brief Вход в программу
 *
 * @param [in] argc  Количество аргументов командной строки
 * @param [in] argv  Значения аргументов командной строки
 *
 * @returns 0 если всё прошло успешно
 */
int main(int argc, char *argv[])
{
    if( argc != 2 )
    {
        fprintf(stderr, "x: ошибка: не задан входной файл\n");
        return 1;
    }

    int     res;    /* Результат работы программы */
    FILE    * f;    /* Входной файл */

    f = fopen(argv[1], "r");

    if( f == NULL )
    {
        fprintf(stderr, "x: ошибка: не удалось открыть входной файл `%s'\n",
                argv[1]);
        return 2;
    }

    yyin    = f;
    setModuleName((strrchr(argv[1], '/')!=NULL)?
                     strrchr(argv[1], '/') + 1:
		     argv[1]);

    Node_t  * node; /* Узел модуля, от которого всё растёт */

    node = node_CreateModuleNode(argv[1]);
    yyparse(node);
    fclose(f);

    res = isGotError();

    if( !res )
    {
        /* Дерево разбора печатаем только если не было ошибок */
        node_PrintNode(node);
    }
    node_DeleteNode(node);

    return res;
}
