#!/bin/bash

# Скрипт запуска тестирования
# Автор Маркин Алексей (alexanius.ru)

# Первый аргумент скрипта - имя тестируемого файла
BIN=$1

# Настраиваем переменные тестирования
INPUT_CORRECT_DIR="tests/in/correct"
SAMPLE_CORRECT_DIR="tests/out/correct"
OUTPUT_CORRECT_DIR="tmp/out/correct"
INPUT_INCORRECT_DIR="tests/in/incorrect"
SAMPLE_INCORRECT_DIR="tests/out/incorrect"
OUTPUT_INCORRECT_DIR="tmp/out/incorrect"

# Проверяем что тестируемый файл существует
if [[ $BIN == "" ]] ;
then
    echo "Ошибка: не задан тестируемый исполняемый файл."
    echo "        Тестируемый файл должен быть задан первым аргументом"
    exit 1
fi

# Проверяем что тестируемый файл исполняемый
if [[ ! -x $BIN ]] ;
then
    echo "Ошибка: тестируемый файт $BIN не является исполняемым"
    exit 2
fi

RES=0

# Тестируем правильные примеры
echo "Тесты верных ситуаций"
mkdir -p $OUTPUT_CORRECT_DIR
for i in `ls "$INPUT_CORRECT_DIR"` ;
do
    echo -n "Тест $i: "

    $BIN "$INPUT_CORRECT_DIR/$i" >& $OUTPUT_CORRECT_DIR/"$i.sample"
    if [ `echo $?` != 0 ]
    then
    echo "ошибка в запуске: $BIN $INPUT_CORRECT_DIR/$i"
    RES=3
        continue
    fi

    diff -q "$OUTPUT_CORRECT_DIR/$i.sample" "$SAMPLE_CORRECT_DIR/$i.sample"
    if [ `echo $?` != 0 ]
    then
        RES=4
        echo "ошибка в запуске: diff -q "$OUTPUT_CORRECT_DIR/$i.sample" "$SAMPLE_CORRECT_DIR/$i.sample""
    else
        echo "пройден!"
    fi
done

# Тестируем неправильные примеры
echo "Тесты ошибочных ситуаций"
mkdir -p $OUTPUT_INCORRECT_DIR
for i in `ls "$INPUT_INCORRECT_DIR"` ;
do
    echo -n "Тест $i: "

    $BIN "$INPUT_INCORRECT_DIR/$i" >& $OUTPUT_INCORRECT_DIR/"$i.sample"
    if [ `echo $?` == 0 ]
    then
    echo "ошибка в запуске: $BIN $INPUT_INCORRECT_DIR/$i"
    RES=5
        continue
    fi

    diff -q "$OUTPUT_INCORRECT_DIR/$i.sample" "$SAMPLE_INCORRECT_DIR/$i.sample"
    if [ `echo $?` != 0 ]
    then
        RES=6
        echo "ошибка в запуске: diff -q "$OUTPUT_INCORRECT_DIR/$i.sample" "$SAMPLE_INCORRECT_DIR/$i.sample""
    else
        echo "пройден!"
    fi
done

# Тестируем ошибки в подаче аргументов

echo "Тесты передаваемых аргументов"

$BIN >& /dev/null
if [ `echo $?` != 1 ]
then
echo "ошибка в запуске: $BIN"
RES=6
fi

$BIN "$OUTPUT_INCORRECT_DIR/!!!" >& /dev/null
if [ `echo $?` != 2 ]
then
echo "ошибка в запуске: $BIN"
RES=6
fi

exit $RES

